/**
* tca.h - P2PWNC Trusted Central Authority (TCA) module
* Copyright (C) 2004-2005 Pantelis Frangoudis, Elias Efstathiou
*
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

#ifndef _PWNC_TCA_H
#define _PWNC_TCA_H

#include "crypt.h"
#include "threads.h"
#include "rdb_engine.h"
#include "conffile.h"
#include "io.h"

#ifndef TEAM_NUM
#define TEAM_NUM 500
#endif

#define DEF_CONF_PATH "../conf/p2pwnc_tca.conf"
#define DEF_CONF_FNAME "p2pwnc_tca.conf"

#define RDB_FILE "../data/receipts.db"
#define RDB_FREE_LIST "../data/receipts.fl"

#define TEAMS_DIR "../data/teams"

//pwnc message codes
#define PWNC_TCA_QUERY 100
#define PWNC_TCA_RECEIPT 110
#define PWNC_OK 120
#define PWNC_GOT_RECEIPT 130
#define PWNC_RECEIPT 140
#define PWNC_GRANT_ACCESS 150
#define PWNC_FORBID_ACCESS 160

//Error codes returned by the receipt verification routine
#define E_DGST 10
#define E_CERT 20
#define E_RCPT 30
#define E_RCPT_EXISTS 40
#define E_TEAM_LOOKUP 50
//Return value: "receipt verified"
#define V_SUCCESS 1

//Socket descriptors:
int tca_sockfd;

//Socket address
struct sockaddr_in tca_addr;

void request_handler(THREAD_INFO *ti);

//defs and funcs below deal with adding/removing teams (mq communication)
#include <sys/types.h>
#include <sys/ipc.h>
#include <sys/msg.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <dirent.h>
#include <signal.h>

#define MTYPE_ADD_TEAM 10 //message type: handle team insertion/deletion
#define MTYPE_DEL_TEAM 20

#define MAX_MSG_LEN 4096

#ifndef DEF_POOL_SIZE
#define DEF_POOL_SIZE 50
#endif

#ifndef DEF_MQ_KEY
#define DEF_MQ_KEY 321
#endif

int mq_id;

//message buffer - this msg will be sent to the tca  module in case a new team has to be added (or an old team removed)
typedef struct msgbuf_t {
	long mtype;
	char mdata[1];
} MSG_BUF;

int init_mq();
void team_handler();

key_t mq_key = DEF_MQ_KEY;
int max_threads;
int server_port = PWNC_TCA_PORT; //default port
unsigned char teams_dir[256];
unsigned char rdb_file[256];
unsigned char rdb_free_list[256];
unsigned char conf_file_path[256];

#endif /*_ PWNC_TCA_H */
